@@ -110,9 +110,9 @@ void    (*__xdm_SessionExit)(struct display *d, int status, int removeAuth) = NU
 void    (*__xdm_DeleteXloginResources)(struct display *d, Display *dpy) = NULL;
 int     (*__xdm_source)(char **environ, char *file) = NULL;
 char    **(*__xdm_defaultEnv)(void) = NULL;
-char    **(*__xdm_setEnv)(char **e, char *name, char *value) = NULL;
+char    **(*__xdm_setEnv)(char **e, const char *name, const char *value) = NULL;
 char    **(*__xdm_putEnv)(const char *string, char **env) = NULL;
-char    **(*__xdm_parseArgs)(char **argv, char *string) = NULL;
+char    **(*__xdm_parseArgs)(char **argv, const char *string) = NULL;
 void    (*__xdm_printEnv)(char **e) = NULL;
 char    **(*__xdm_systemEnv)(struct display *d, char *user, char *home) = NULL;
 void    (*__xdm_LogOutOfMem)(const char * fmt, ...) = NULL;
@@ -194,7 +194,7 @@ GreetDone (
     LoginData	*data,
     int		status)
 {
-    Debug ("GreetDone: %s, (password is %d long)\n",
+    Debug ("GreetDone: %s, (password is %zu long)\n",
 	    data->name, strlen (data->passwd));
     switch (status) {
     case NOTIFY_OK:
